# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

import json
import sys
import traceback
import sha.config

from sha.contrib.metric.edge_base import EdgeBase
from sha.contrib.metric.utils import timeout_cache

class EdgeLRPortStats(EdgeBase):
    def get_lr_port_stats(self):
        lr_dict = {}
        try:
            lr_dict = self.get_lr_list(['SERVICE_ROUTER_TIER0', 'SERVICE_ROUTER_TIER1', 'VRF_SERVICE_ROUTER_TIER0'])
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get edge fw interface for T0/T1 lrouter: {}, {}".format(e, trace))
        finally:
            if not lr_dict:
                return None

        lr_port_stats = []
        lr_port_stats_dict = {}
        # Get LR port stats
        # The cost of nsxcli is much higher than edge backend command
        #     """nsxcli -c 'get logical-router interfaces stats'"""
        # Get LR interface stats via edge backend command
        cmd_get_stats = ['edge-appctl', '-t', '/var/run/vmware/edge/dpd.ctl', 'lrouter_port/show']
        output = self.run_command_with_time_check(cmd_get_stats)
        lr_port_stats = json.loads(output)

        for stat in lr_port_stats:
            lr = lr_dict.get(stat['lrouter'], None)
            if lr is not None:
                stat['mp_router_id'] = lr['mp_router_id']
                stat['lr_type'] = lr['type']
            lr_port_stats_dict[stat["ifuuid"]] = stat

        self._logger.debug("edge lr port stats({}): {}".format(sys.getsizeof(lr_port_stats), lr_port_stats))
        return lr_port_stats_dict

    @timeout_cache(10)
    def usage(self, *args, **kwargs):
        lr_port_stats = {}
        try:
            lr_port_stats = self.get_lr_port_stats()
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get edge lr port stats {}, {}".format(e, trace))
            lr_port_stats = {}
        return {'lr_port_stats' : lr_port_stats}
