import re
import time
import sha.config as conf
from sha.contrib.metric.abc_base import Metric



class EdgeRcuBlock(Metric):
    def __init__(self):
        self._logger = conf.get_logger(__name__)
        self.start_offset = 0
        self.end_offset = 0
        self.active_events = dict()
        self.pattern = re.compile('WARN.*blocked (\d+) ms.*for ([^\s]+) to quiesce$')

    def usage(self, *args, **kwargs):
        if not conf.node_type_is(conf.DIST_EDGE,
                                 conf.DIST_AUTONOMOUS_EDGE,
                                 conf.DIST_PUBLIC_CLOUD_GATEWAY):
            return None
        detected_events = []
        with open('/var/log/syslog', 'r') as logfile:
            logfile.seek(0, 2)
            self.end_offset = logfile.tell()
            if self.start_offset > self.end_offset:
                self.start_offset = 0
            logfile.seek(self.start_offset, 0)

            if self.end_offset > self.start_offset:
                for line in logfile.readlines():
                    if logfile.tell() > self.end_offset:
                        break
                    match = re.search(self.pattern, line)
                    if match:
                        blocked_time = int(match.group(1))
                        thread_name = str(match.group(2))
                        if blocked_time > 120000:
                            detected_events.append(str(thread_name))
                            if thread_name in self.active_events:
                                self.active_events[thread_name]['ref_cnt'] += 1
                            else:
                                self.active_events[thread_name] = dict()
                                self.active_events[thread_name]['ref_cnt'] = 1
                                self.active_events[thread_name]['event_state'] = True
                                self._logger.warn("The thread %s is blocked", thread_name)

            self.start_offset = self.end_offset

            for key in self.active_events:
                if key not in detected_events:
                    self.active_events[key]['ref_cnt'] -= 1
                    if self.active_events[key]['ref_cnt'] == 0:
                        self.active_events[key]['event_state'] = False
                        self._logger.info("The thread %s is unblocked", str(key))

            return self.active_events
